#!/bin/sh

# >>> correct these variable before using! <<<
VERSION=0-5
# this folder must contain texpages and tileset subfolders
BASEDIR="$HOME/wz/mods/renew/source"
# a place to put temporary files that will be deleted on script exit
TEMPDIR="/tmp"
# >>>  end of adjustable variables list.  <<<

TEXDIR=$BASEDIR/texpages
WATERTILENUMBER=17

mkdir -pv texpages

function convertTile {
	if [ "`echo $1 | sed s/tileset//g | grep tiles`" != "" ]; then
		convert -crop 128x128+0+0 $1 $2
	else
		convert -resize 128 $1 $2
	fi
}

function composeTile {
	convertTile $1 $TEMPDIR/c0.png
	convertTile $2 $TEMPDIR/c1.png
	convertTile $3 $TEMPDIR/c2.png
	convertTile $4 $TEMPDIR/c3.png
	composite -compose Copy_Opacity $TEMPDIR/gradient-0.png   $TEMPDIR/c0.png $TEMPDIR/0.png
	composite -compose Copy_Opacity $TEMPDIR/gradient-90.png  $TEMPDIR/c1.png $TEMPDIR/1.png
	composite -compose Copy_Opacity $TEMPDIR/gradient-270.png $TEMPDIR/c2.png $TEMPDIR/2.png
	composite -compose Copy_Opacity $TEMPDIR/gradient-180.png $TEMPDIR/c3.png $TEMPDIR/3.png
	composite -compose plus $TEMPDIR/0.png $TEMPDIR/1.png $TEMPDIR/a.png
	composite -compose plus $TEMPDIR/a.png $TEMPDIR/2.png $TEMPDIR/b.png
	composite -compose plus $TEMPDIR/b.png $TEMPDIR/3.png $5
}

function composeDecalTile {
	composeTile $1 $2 $3 $4 $TEMPDIR/c.png
	composite $5 $TEMPDIR/c.png $6
}

function composeWaterTile {
	composeTile $1 $2 $3 $4 $TEMPDIR/c.png
	composite -compose Copy_Opacity $5 $TEMPDIR/c.png $6
}

function getTexture {
	page=`grep $1, $gtypeFile | tr "," " " | awk '{print $2}'`
	#if [ "`echo $1 | grep water`" != "" ]; then
		#page="page-80"
	#fi
	ls $TEXDIR/$page*
}

echo Preparing the gradients ...
convert -rotate 0   gradient.png $TEMPDIR/gradient-0.png
convert -rotate 90  gradient.png $TEMPDIR/gradient-90.png
convert -rotate 180 gradient.png $TEMPDIR/gradient-180.png
convert -rotate 270 gradient.png $TEMPDIR/gradient-270.png

for i in $*; do

	if [ "$i" = "1" ]; then
		tilesetName="arizona"
	elif [ "$i" = "2" ]; then
		tilesetName="urban"
	elif [ "$i" = "3" ]; then
		tilesetName="rockie"
	fi

	modName=fancy-${tilesetName}-${VERSION}.wz

	echo Making $tilesetName tileset ...

	groundFile=$BASEDIR/tileset/${tilesetName}ground.txt
	gtypeFile=$BASEDIR/tileset/tertilesc${i}hwGtype.txt
	decalList=$BASEDIR/tileset/${tilesetName}decals.txt
	decalDir=$TEXDIR/tertilesc${i}hw-128
	waterTile=$decalDir/tile-$WATERTILENUMBER.png

	for j in 16 32 64 128; do
		rm -r $TEMPDIR/result > /dev/null 2>&1
		mkdir $TEMPDIR/result
		rm -r texpages/tertilesc${i}hw-$j > /dev/null 2>&1
		mkdir texpages/tertilesc${i}hw-$j
	done

	num=0
	grep -v $tilesetName "$groundFile" | tr "," " " | while read a b c d; do

		znum=$num
		if [ $znum -lt 10 ]; then
			znum=0$znum
		fi
		destFile=$TEMPDIR/result/tile-$znum.png

		A=`getTexture $a`
		B=`getTexture $b`
		C=`getTexture $c`
		D=`getTexture $d`
		decalTile=$decalDir/tile-$znum.png
		if [ "$znum" = "$WATERTILENUMBER" ]; then
			cp $decalTile $destFile
			for j in 16 32 64 128; do 
				convert -resize $j $destFile PNG32:texpages/tertilesc${i}hw-$j/tile-$znum.png
			done
		else
			if [ "`echo $a $b $c $d | grep water`" != "" ]; then
				composeWaterTile $A $B $C $D $decalTile $destFile
			else
				decal=""
				if [ "`grep ^$znum $decalList`" != "" ]; then
					decal=$decalTile
				fi
				if [ -e decals/$i/tile-$znum.png ]; then
					decal=decals/$i/tile-$znum.png
				fi
				if [ "$decal" = "" ]; then
					composeTile $A $B $C $D $destFile
				else
					composeDecalTile $A $B $C $D $decal $destFile
				fi
			fi
			for j in 16 32 64 128; do 
				k=$(( $j/16-1 ))
				composite -geometry $(( $j-2*$k ))x$(( $j-2*$k ))+$k+$k -size $j $destFile empty${j}.png PNG8:texpages/tertilesc${i}hw-$j/tile-$znum.png
			done
		fi
		((num++))

	done

	echo Zipping up the $tilesetName mod ...

	rm -v $modName > /dev/null 2>&1
	zip $modName texpages/tertilesc${i}hw-* -r > /dev/null 2>&1

done

echo Cleaning up ...
TEMPFILES="0.png 1.png 2.png 3.png a.png b.png c.png gradient-0.png gradient-90.png gradient-180.png gradient-270.png c0.png c1.png c2.png c3.png"
for i in $TEMPFILES; do 
	rm $TEMPDIR/$i
done
rm -r $TEMPDIR/result